@@ -1,83 +1,70 @@
 package Debug::Client;
 
-use 5.008006;
+use 5.010;
 use strict;
-use warnings;
+use warnings FATAL => 'all';
 
-our $VERSION = '0.20';
+use English qw( -no_match_vars );
+local $OUTPUT_AUTOFLUSH = 1;
+
+our $VERSION = '0.22';
 
 use utf8;
-use IO::Socket;
-use Carp qw(carp croak cluck);
+use IO::Socket::IP 0.18;
+use Carp 1.20 qw(carp croak);
 
 use constant {
 	BLANK => qq{ },
+	NONE  => q{},
 };
 
-# use Data::Printer { caller_info => 1, colored => 1, };
 
 #######
 # new
 #######
 sub new {
-	my $class = shift;   # What class are we constructing?
-	my $self  = {};      # Allocate new memory
-	bless $self, $class; # Mark it of the right type
-	$self->_init(@_);    # Call _init with remaining args
+	my ( $class, @args ) = @_; # What class are we constructing?
+	my $self = {};             # Allocate new memory
+	bless $self, $class;       # Mark it of the right type
+	$self->_initialize(@args); # Call _initialize with remaining args
 	return $self;
 }
+
 #######
-# _init
+# _initialize
 #######
-sub _init {
+sub _initialize {
 	my ( $self, %args ) = @_;
 
-	$self->{local_host} = $args{host} ? $args{host} : 'localhost';
-	$self->{local_port} = $args{port} ? $args{port} : 24642;
+	$self->{local_host} = $args{host} // 'localhost';
+	$self->{local_port} = $args{port} // 24_642;
 
-	#ToDo for IO::Socket::IP
-	# $self->{porto}      = $args{porto}  ? $args{porto}  : 'tcp';
-	# $self->{listen}     = $args{listen} ? $args{listen} : SOMAXCONN;
-	# $self->{reuse_addr} = $args{reuse}	? $args{reuse}  : 1;
+	#for IO::Socket::IP
+	$self->{porto}      = $args{porto}  // 'tcp';
+	$self->{listen}     = $args{listen} // 1;
+	$self->{reuse_addr} = $args{reuse}  // 1;
 
 	$self->{buffer} = undef;
 	$self->{module} = undef;
 
-	return;
-}
-
-#######
-# Method listener
-#######
-sub listener {
-	my $self = shift;
-
 	# Open the socket the debugger will connect to.
-	my $sock = IO::Socket::INET->new(
+	my $sock = IO::Socket::IP->new(
 		LocalHost => $self->{local_host},
 		LocalPort => $self->{local_port},
-		Proto     => 'tcp',
-		Listen    => SOMAXCONN,
-		ReuseAddr => 1,
-
-		# Proto     => $self->{porto},
-		# Listen    => $self->{listen},
-		# ReuseAddr => $self->{reuse_addr},
-	);
-	$sock or carp "Could not connect to '$self->{local_host}' '$self->{local_port}' no socket :$!";
-	_logger("listening on '$self->{local_host}:$self->{local_port}'");
-
-	$self->{sock}     = $sock;
-	$self->{new_sock} = $self->{sock}->accept();
+		Proto     => $self->{porto},
+		Listen    => $self->{listen},
+		ReuseAddr => $self->{reuse_addr},
+	) or carp "Could not connect to '$self->{local_host}' '$self->{local_port}' no socket :$!";
 
+	$self->{socket} = $sock->accept();
 	return;
 }
 
 
 #######
-# Method buffer
+# Method get_buffer
 #######
-sub buffer {
+sub get_buffer {
 	my $self = shift;
 
 	return $self->{buffer};
@@ -113,12 +100,13 @@ sub get_lineinfo {
 
 	$self->_send('.');
 	$self->_get;
-	$self->{buffer} =~ m{^[\w:]*				# module
-						(?:CODE\(.*\))* 		# catch CODE(0x9b434a8)
-						\( ([^\)]*):(\d+) \)	# (file):(row)
-                                    }mx;
-	$self->{filename} = $1;
-	$self->{row}      = $2;
+	$self->{buffer} =~ m{
+		^[\w:]*                             # module
+		(?:CODE[(].*[)])*                   # catch CODE(0x9b434a8)
+		[(] (?<file>[^\)]*):(?<row>\d+) [)] # (file):(row)
+	}smx;
+	$self->{filename} = $+{file};
+	$self->{row}      = $+{row};
 
 	return;
 }
@@ -237,23 +225,27 @@ sub set_breakpoint {
 	$self->_prompt;
 
 	# if it was successful no reply
-
-	if ( $self->{buffer} =~ /^Subroutine [\w:]+ not found\./ ) {
-
-		# failed
-		return 0;
-	} elsif ( $self->{buffer} =~ /^Line \d+ not breakable\./ ) {
-
-		# failed to set on line number
-		return 0;
-	} elsif ( $self->{buffer} =~ /^\d+ levels deep in subroutine calls!/ ) {
-
-		# failed
-		return 0;
-	} elsif ( $self->{buffer} =~ /\S/ ) {
-		return 0;
+	given ( $self->{buffer} ) {
+		when ( $_ =~ /^Subroutine [\w:]+ not found[.]/sxm ) {
+			return 0;
+		}
+		when ( $_ =~ /^Line \d+ not breakable[.]/sxm ) {
+			return 0;
+		}
+		when ( $_ =~ /^\d+ levels deep in subroutine calls!/sxm ) {
+			return 0;
+		}
+		when ( $_ =~ /^Already in/m ) {
+			return 1;
+		}
+		when ( $_ =~ /\S/sxm ) {
+			# say 'Non-whitespace charter found';
+			return 0;
+		}
+		default {
+			return 1;
+		}
 	}
-	return 1;
 }
 
 #######
@@ -281,10 +273,11 @@ sub show_breakpoints {
 	$self->_send('L');
 	$self->_get;
 	$self->_prompt;
+
 	return $self->{buffer};
 }
 
-# TODO if the given $x is a reference then something (either this module or its user) should actually call x $var
+
 #######
 # Accessor get_value
 #######
@@ -296,7 +289,7 @@ sub get_value {
 		$self->_get;
 		$self->_prompt;
 		return $self->{buffer};
-	} elsif ( $var =~ /\@/ or $var =~ /\%/ ) {
+	} elsif ( $var =~ /^\@/sxm or $var =~ /^\%/sxm ) {
 		$self->_send("x \\$var");
 		$self->_get;
 		$self->_prompt;
@@ -305,7 +298,7 @@ sub get_value {
 		$self->_send("p $var");
 		$self->_get;
 		$self->_prompt;
-		if ( $self->{buffer} =~ m/^(?:HASH|ARRAY)/ ) {
+		if ( $self->{buffer} =~ m/^(?:HASH|ARRAY)/sxm ) {
 			$self->_send("x \\$var");
 			$self->_get;
 			$self->_prompt;
@@ -335,7 +328,17 @@ sub get_p_exp {
 sub get_y_zero {
 	my $self = shift;
 
-	$self->_send("y 0");
+	require PadWalker if 0; #forces PadWalker to be a requires not a test_requires
+
+	# say 'running on perl '. $PERL_VERSION;
+	if ( $PERL_VERSION >= 5.017006 ) {
+		# say 'using y=1 instead as running on perl ' . $PERL_VERSION;
+		$self->_send('y 1');
+	} else {
+		$self->_send('y 0');
+	}
+
+	# $self->_send('y 0');
 	$self->_get;
 	$self->_prompt;
 
@@ -382,7 +385,10 @@ sub get_x_vars {
 #######
 sub get_h_var {
 	my ( $self, $var ) = @_;
-
+	
+	#added a flush buffer to stop help appending in an initional case
+	$self->{buffer} = undef;
+	
 	if ( defined $var ) {
 		$self->_send("h $var");
 	} else {
@@ -391,8 +397,8 @@ sub get_h_var {
 
 	$self->_get;
 
-	#Tidy for Output Panel
-	$self->{buffer} =~ s/(\e\[4m|\e\[24m|\e\[1m|\e\[0m)//mg;
+	#Tidy for Padre Output Panel
+	$self->{buffer} =~ s/(\e\[4m|\e\[24m|\e\[1m|\e\[0m)//sxmg;
 	$self->_prompt;
 
 	return $self->{buffer};
@@ -404,7 +410,8 @@ sub get_h_var {
 sub set_option {
 	my ( $self, $option ) = @_;
 
-	unless ( defined $option ) {
+	# unless ( defined $option ) {
+	if ( not defined $option ) {
 		return 'missing option';
 	}
 
@@ -437,26 +444,26 @@ sub get {
 	if (wantarray) {
 		$self->_prompt;
 		my ( $module, $file, $row, $content ) = $self->_process_line;
-
 		return ( $module, $file, $row, $content );
 	} else {
+
 		return $self->{buffer};
 	}
 }
 
 #######
-# Method filename
+# Method get_filename
 #######
-sub filename {
+sub get_filename {
 	my $self = shift;
 
 	return $self->{filename};
 }
 
 #######
-# Method row
+# Method get_row
 #######
-sub row {
+sub get_row {
 	my $self = shift;
 
 	return $self->{row};
@@ -480,36 +487,26 @@ sub module {
 sub _get {
 	my $self = shift;
 
-	#my $remote_host = gethostbyaddr($sock->sockaddr(), AF_INET) || 'remote';
-	my $buffer = q{};
-	my $ret;
+	my $buffer = NONE;
+
 	while ( $buffer !~ /DB<\d+>/ ) {
-		$ret = $self->{new_sock}->sysread( $buffer, 1024, length $buffer );
+		my $ret = $self->{socket}->sysread( $buffer, 1024, length $buffer );
+
 		if ( not defined $ret ) {
 			carp $!; # TODO better error handling?
 		}
 
-		# _logger("---- ret '$ret'\n$buffer\n---");
 		if ( not $ret ) {
 			last;
 		}
 	}
-	_logger("---- ret $ret\n$buffer\n---");
-	_logger("_get done");
 
 	$self->{buffer} = $buffer;
 
-	return $buffer;
-}
-
-#######
-# Internal Method _logger
-#######
-sub _logger {
-	print "LOG: $_[0]\n" if $ENV{DEBUG_LOGGER};
 	return;
 }
 
+
 #######
 # Internal Method _process_line
 #######
@@ -524,21 +521,15 @@ sub _logger {
 #    $content   is the content of the current row
 # see 00-internal.t for test cases
 sub _process_line {
-	my $self = shift;
+	my $self   = shift;
 	my $buffer = $self->{buffer};
-	
-	# print "inside _process_line\n";
-	
+
 	my $line    = BLANK;
 	my $module  = BLANK;
 	my $file    = BLANK;
 	my $row     = BLANK;
 	my $content = BLANK;
 
-	# if ( not defined $buffer or not ref $buffer or ref $buffer ne 'SCALAR' ) {
-	# carp('_process_line should be called with a reference to a scalar');
-	# }
-
 	if ( $buffer =~ /Debugged program terminated/ ) {
 		$module = '<TERMINATED>';
 		$self->{module} = $module;
@@ -555,35 +546,40 @@ sub _process_line {
 		croak("Debug::Client: Line is undef. Buffer is  $self->{buffer}");
 	}
 
-	# _logger("Line1: $line");
 	my $cont = 0;
 	if ($line) {
-		if ( $line =~ /^\d+:   \s*  (.*)$/x ) {
+		if ( $line =~ /^\d+: \s* (.*)$/x ) {
 			$cont = $1;
 			$line = pop @parts;
 
 		}
 	}
 
-	if ($line =~ m{^([\w:]*) 			# module
-                  \( ([^\)]*):(\d+) \) 	# (file:row)
-                  :\t? 					# :
-                  (.*) 					# content
+	if ($line =~ m{^(?<module>[\w:]*)                 # module
+                  [(] (?<file>[^\)]*):(?<row>\d+) [)] # (file:row)
+                  :\t?                                # :
+                  (?<content>.*)                      # content
                   }mx
 		)
 	{
-		( $module, $file, $row, $content ) = ( $1, $2, $3, $4 );
+		( $module, $file, $row, $content ) = ( $+{module}, $+{file}, $+{row}, $+{content} );
 	}
-	if ( $module eq BLANK || $file eq BLANK || $row eq BLANK ) {
 
-		# unless ( defined $module || defined $file || defined $row ) {
-		my $current_file = $self->show_line();
+	# if ( $module eq BLANK || $file eq BLANK || $row eq BLANK ) {
+	# we did not need to test for everthing
+	if ( $module eq BLANK ) {
 
-		$current_file =~ m/([\w:]*) \( (.*) : (\d+) .* /mgx;
+		# preserve buffer why we check where we are test_1415.pl
+		my $preserve_buffer = $self->{buffer};
+		my $current_file    = $self->show_line();
 
-		$module = $1;
-		$file   = $2;
-		$row    = $3;
+		# $current_file =~ m/([\w:]*) \( (.*) : (\d+) .* /mgx;
+		$current_file =~ m/(?<module>[\w:]*) [(] (?<file>.*) : (?<row>\d+) .* /mgxs;
+
+		$module         = $+{module};
+		$file           = $+{file};
+		$row            = $+{row};
+		$self->{buffer} = $preserve_buffer;
 
 	}
 
@@ -609,21 +605,15 @@ sub _process_line {
 # See 00-internal.t for test cases
 sub _prompt {
 	my $self = shift;
-	# my $buffer = $self->{buffer};
 	
-	# print "inside _prompt\n";
-
 	my $prompt;
-	if ( $self->{buffer} =~ s/\s*DB<(\d+)>\s*$// ) {
-		$prompt = $1;
-		_logger("prompt: $prompt");
+	if ( $self->{buffer} =~ s/\s*DB<(?<prompt>\d+)>\s*$// ) {
+		$prompt = $+{prompt};
 	}
 
 	chomp $self->{buffer};
 	$self->{prompt} = $prompt;
-	# $self->{buffer} = $buffer;
-	# p $self->{prompt};
-	# p $self->{buffer};
+
 	return $self->{prompt};
 }
 
@@ -633,13 +623,14 @@ sub _prompt {
 sub _send {
 	my ( $self, $input ) = @_;
 
-	print { $self->{new_sock} } "$input\n";
+	$self->{socket}->print( $input . "\n" );
 
 	return 1;
 }
 
 #######
 # Internal Method _send_get
+# send then get
 #######
 sub _send_get {
 	my ( $self, $input ) = @_;
@@ -682,19 +673,20 @@ __END__
 
 =pod
 
+=encoding utf8
+
 =head1 NAME
 
 Debug::Client - debugger client side code for Padre, The Perl IDE.
 
 =head1 VERSION
 
-This document describes Debug::Client version 0.20
+This document describes Debug::Client version 0.22
 
 =head1 SYNOPSIS
 
   use Debug::Client;
   my $debugger = Debug::Client->new(host => $host, port => $port);
-  $debugger->listener;
 
 Where $host is the host-name to be used by the script under test (SUT)
 to access the machine where Debug::Client runs. If they are on the same machine
@@ -731,11 +723,11 @@ Once the script under test was launched we can call the following:
 
   $debugger->execute_code( '@name = qw(foo bar)' );
 
-  my $value = $debugger->get_value('@name');  $value is the dumped data?
+  my $value = $debugger->get_value('@name'); # $value is the dumped data?
 
   $debugger->execute_code( '%phone_book = (foo => 123, bar => 456)' );
 
-  my $value = $debugger->get_value('%phone_book');  $value is the dumped data?
+  my $value = $debugger->get_value('%phone_book'); # $value is the dumped data?
   
   
   $debugger->set_breakpoint( "file", 23 ); # set breakpoint on file, line
@@ -771,9 +763,11 @@ Once the script under test was launched we can call the following:
 
 =head1 DESCRIPTION
 
-The prime use of this module is to provide debugger functionality for Padre 0.94+, 
+This is a DEVELOPMENT Release only, you have been warned!
 
-This module should be Perl 5.16.0 ready.
+The primary use of this module is to provide debugger functionality for Padre 0.98 and beyond, 
+
+This module has been tested against Perl 5.17.8.
 
 =head1 METHODS
 
@@ -786,28 +780,12 @@ The constructor can get two parameters: host and port.
   my $debugger = Debug::Client->new;
 
   my $debugger = Debug::Client->new(host => 'remote.host.com', port => 24642);
-   
-Immediately after the object creation one needs to call
 
-  $debugger->listener;
-  
-TODO: Is there any reason to separate the two?
-
-
-
-=item listener
-
-listener/hearken To listen attentively; give heed. See C<BUGS AND LIMITATIONS>
-
-See C<new>
-
- $debugger->listener
-
-=item buffer
+=item get_buffer
 
 Returns the content of the buffer since the last command
 
-  $debugger->buffer;
+  $debugger->get_buffer;
 
 =item quit
 
@@ -831,8 +809,8 @@ trying to use perl5db line-info in naff way,
 
 Then use the following as and when.
 
- $debugger->filename;
- $debugger->row;
+ $debugger->get_filename;
+ $debugger->get_row;
  
 to get filename and row for ide due to changes in perl5db v1.35 see perl5156delta
 
@@ -949,12 +927,14 @@ From perldebug, but defaulted to y 0
 
 Display all (or some) lexical variables (mnemonic: my variables) in the current 
 scope or level scopes higher. You can limit the variables that you see with vars 
-which works exactly as it does for the V and X commands. Requires the PadWalker 
-module version 0.08 or higher; will warn if this isn't installed. 
+which works exactly as it does for the V and X commands. Requires that the PadWalker 
+module be installed
 Output is pretty-printed in the same style as for V and the format is controlled by the same options.
 
   $debugger->get_y_zero();
 
+which is now y=1 since perl 5.17.6, 
+
 =item get_v_vars
 
 V [pkg [vars]]
@@ -1026,13 +1006,13 @@ $file and $row describe the location of the next instructions.
 $content is the actual line - this is probably not too interesting as it is 
 in the editor. $module is just the name of the module in which the current execution is.
 
-=item filename
+=item get_filename
 
- $debugger->filename();
+ $debugger->get_filename();
 
-=item row
+=item get_row
 
- $debugger->row();
+ $debugger->get_row();
 
 =item module
 
@@ -1046,8 +1026,6 @@ in the editor. $module is just the name of the module in which the current execu
 
 =item * _get
 
-=item * _logger
-
 =item * _process_line
 
 =item * _prompt
@@ -1080,19 +1058,21 @@ and just performing c on it's own
 
 I<Warning sub listen has bean deprecated>
 
-Has bean deprecated since 0.13_04 and all future versions starting with v0.14
+Has bean deprecated since 0.13_04 and all future version starting with v0.14
 
 Perl::Critic Error Subroutine name is a homonym for built-in function
 
 Use $debugger->listener instead
 
+It will work against perl 5.17.6-7 with rindolf patch 7a0fe8d applied for watches
+
 =head1 AUTHORS
 
 Kevin Dawson E<lt>bowtie@cpan.orgE<gt>
 
 Gabor Szabo E<lt>gabor@szabgab.comE<gt>
 
-=head1 CONTRIBUTORS
+=head2 CONTRIBUTORS
 
 Breno G. de Oliveira E<lt>garu at cpan.orgE<gt>
 
@@ -1100,11 +1080,17 @@ Ahmad M. Zawawi E<lt>ahmad.zawawi@gmail.comE<gt>
 
 Mark Gardner E<lt>mjgardner@cpan.orgE<gt>
 
-Wolfram Humann E<lt>whumann@cpan.orgE<gt> 
+Wolfram Humann E<lt>whumann@cpan.orgE<gt>
+
+Adam Kennedy E<lt>adamk@cpan.orgE<gt>
+
+Alexandr Ciornii E<lt>alexchorny@gmail.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2008-2012 Gabor Szabo/Kevin Dawson
+Copyright 2008-2011 Gabor Szabo
+
+Some parts copyright 2011-2013 Kevin Dawson and CONTRIBUTORS as listed above.
 
 =head1 LICENSE
 
